#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/5 4:36 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : config.py
# @Software: PyCharm
# @Desc    :

websocket_address = "localhost"
websocket_port = 8101
websocket_path = "ws/simulator"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/20 11:39 上午
# @Author  : shuming.wsm
# @Site    : 
# @File    : tools.py
# @Software: PyCharm
# @Desc    :
import random
import time


def build_msg_id():
    return int(time.time() * 1000) * 100000 + random.randint(10000, 99999)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/20 12:03 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : main.p.py
# @Software: PyCharm
# @Desc    : demo应用， 如果用户需要本地模拟websocket, 请先启动"simulator.py", 如果实际运行请修改config
import asyncio
import json
import time

from agent.agentClient import AgentClient
from agent.packages.message.base import CmdContent, TextContent
from agent.example.config import *
from agent.example.demoReceiveMessage import DemoReceiveMessage


def message_process_callback(content):
    if isinstance(content, CmdContent):
        print("process cmd content, cmd_code: %s, params:%s" % (content.get_code(), json.dumps(content.get_params())))
    if isinstance(content, TextContent):
        if content.is_compress():
            print("process text content, text is compress")
        else:
            print("process text content, text: %s" % content.get_text())


def initialized():
    print("this is initialized")


async def start_agent_client(agent_client):
    print("Start agent client")
    await asyncio.gather(
        agent_client.agent_run()
    )


if __name__ == "__main__":
    agentClient = AgentClient(websocket_address,
                              websocket_port,
                              websocket_path,
                              "demo",
                              message_process_callback,
                              initialized)
    agentClient.register_message_class("DEMO_MESSAGE", DemoReceiveMessage)
    event_loop = asyncio.get_event_loop()
    asyncio.set_event_loop(event_loop)
    while True:
        try:
            event_loop.run_until_complete(start_agent_client(agentClient))
            asyncio.get_event_loop()
        except Exception as e:
            print("Failed start agent client", e.__str__())
        finally:
            time.sleep(5)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/4 8:51 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : agentManage.py
# @Software: PyCharm
# @Desc    :
import time
import _thread

import websocket

from agent.messageClass import defaultReceiveMessageClassList
from agent.packages.base import Package
from agent.packages.message.base import Message
from agent.packages.message.send.initializedAck import InitializedAck
from agent.logger import logger
from agent.packages.send import PING


class AgentClient(object):
    env = None
    url = None
    websocket_connect = None
    biz_work_id = None
    # 下行消息第三方处理回掉
    message_process_callback = None
    # 消息类型列表
    message_class_list = None
    # 模块名称
    model_name = None
    # 模块初始化回调函数
    initialized_callback = None

    def __init__(self, ws_address,
                 ws_port,
                 url_path,
                 model_name,
                 message_process_callback=None,
                 initialized_callback=None):
        """
        ws_address
        """
        self.url = 'ws://%s:%d/%s' % (ws_address, ws_port, url_path)
        self.message_process_callback = message_process_callback
        self.model_name = model_name
        self.message_class_list = defaultReceiveMessageClassList
        self.initialized_callback = initialized_callback

    def register_message_class(self, name, class_name):
        self.message_class_list[name] = class_name

    def get_cmd_code_message_class(self, cmd_code):
        if cmd_code in self.message_class_list.keys():
            return self.message_class_list[cmd_code]
        else:
            return None

    def on_message(self, ws, message):
        _thread.start_new_thread(self.handle, (message,))

    def on_open(self, ws):
        def run(agent_client):
            try:
                if self.initialized_callback:
                    self.initialized_callback()
            except Exception as e:
                # 如果调用初始化接口失败， 则不发送InitializedAck
                logger.exception("call model initialized_callback exception, %s", e.__str__())
                time.sleep(30)
                return
            # 首次启动初始化成功， 发送ACK
            initialized_ack = InitializedAck(agent_client)
            if initialized_ack.is_valid():
                ws.send(initialized_ack.get_package_string())
            else:
                logger.error("send initialized ack error")
            while True:
                try:
                    # 每10秒发送一个PING包
                    ping_package = PING(agent_client)
                    ping_package.process()
                    time.sleep(10)
                except Exception as e:
                    logger.exception("send heartbeat error, %s", e.__str__())
                    break
                logger.info("send heartbeat")
        _thread.start_new_thread(run, (self,))

    def on_error(self, ws, error):
        logger.error("ws error")

    def on_close(self, ws, close_status_code, close_msg):
        logger.error("websocket had error", close_status_code, close_msg)

    def agent_run(self):
        logger.info('agent run')
        try:
            websocket.enableTrace(False)
            self.websocket_connect = websocket.WebSocketApp(
                self.url,
                on_open=self.on_open,
                on_message=self.on_message,
                on_error=self.on_error,
                on_close=self.on_close
            )
            self.websocket_connect.run_forever()
            logger.error("websocket connect close")
        except Exception as e:
            logger.error('agent run error', e.__str__())

    # 发送数据, 消息协议由业务层保障
    def send_message(self, message):
        if self.websocket_connect:
            self.websocket_connect.send(message)

    def send_text_message(self, header, body):
        text_message = Message(header, body)
        if text_message.is_valid():
            self.send_message(text_message.get_package_string())

    def handle(self, data):
        if len(data) == 0:
            logger.error("process message error, data is None")
            return

        try:
            package = Package(data, self)
            if package.is_valid():
                package.process()
        except Exception as e:
            logger.exception("package error, %s" % e.__str__())
            return




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/8 7:43 下午
# @Author  : shuming.wsm
# @Site    :
# @File    : protocol.py
# @Software: PyCharm
# @Desc    : Agent 1.0 协议
import json
import random
import time

messageVersion = '1.0'

msgTarget = [
    'SYSTEM',  # 系统消息
    'INNER',  # 内部消息
    'OUTER',  # 外部消息
    'AGENT'  # 与agent指令通讯
    "STATELESS_OUTER"
]

messageTypeList = [
    'cmd',  # 以JSON格式进行传输
    'text'  # 以字符串的形式进行传输
]


class Header(object):
    """
    msgId: 消息id 13位时间戳(毫秒)+5位随机数, String , 必传
    msgTarget: 消息投递目标, String , 必传
       SYSTEM: 系统消息，发送给资源调度系统   eg.异常上报
       INNER: 内部消息，发送给其他容器
       OUTER 外部消息，发给业务系统
       AGENT 与agent指令通讯
       STATELESS_OUTER 常驻外部消息通道, 业务定制预留通道， 容器内部不需要使用
    receiver: 接收者  当 msgTarget=INNER 时. 必传 , String
    sender: 发送者 String , 非必传
    version: 消息协议版本, 默认1.0 String 必传
    outerType: 外部消息接收者类型 String, 非必传. 外部消息时使用  msgTarget= OUTER ， 当msgTarget为STATELESS_OUTER时必传。
        server: 发送给外部服务短业务系统
        client：发送给client, 比如H5
    """
    headerData = None
    msgId = None
    msgTarget = None
    receiver = None
    sender = None
    version = None
    outerType = None

    def __init__(self, header_data):
        self.headerData = header_data

    def get_header_data(self):
        return self.headerData

    def is_valid(self):
        header_key_list = [
            {
                "key": "msgId",
                "mustInput": True,
                "type": int
            },
            {
                "key": "msgTarget",
                "mustInput": False,
                "type": str
            },
            {
                "key": "receiver",
                "mustInput": False,
                "type": str
            },
            {
                "key": "sender",
                "mustInput": True,
                "type": str
            },
            {
                "key": "version",
                "mustInput": False,
                "type": str
            },
            {
                "key": "outerType",
                "mustInput": False,
                "type": str
            },
        ]
        h_keys = self.headerData.keys()
        for header_key in header_key_list:
            if header_key['mustInput']:
                if header_key["key"] in h_keys:
                    key = header_key['key']
                    if isinstance(self.headerData[key], header_key['type']):
                        setattr(self, key, self.headerData[key])
                    else:
                        raise Exception("%s not %s, please check it" %(header_key['key'], header_key['type']))
                else:
                    raise Exception("%s not exist" % header_key['key'])
        if self.version:
            if self.version != messageVersion:
                raise Exception("message version error, current message version:%s" % self.version)
        else:
            self.version = messageVersion
            self.headerData['version'] = messageVersion
        if self.msgTarget:
            if self.msgTarget not in msgTarget:
                raise Exception("msgTarget error, must is '%s', \
                    but you in %s" % (msgTarget.__str__(), self.msgTarget))
            if msgTarget == "INNER":
                if not self.receiver:
                    raise Exception("msgTarget is 'INNER', receiver must input!")
            if msgTarget == "OUTER" or msgTarget == "STATELESS_OUTER":
                if not self.outerType:
                    raise Exception("msgTarget is 'OUTER', outerType must input!")
                if self.outerType not in ['server', 'client']:
                    raise Exception("msgTarget is 'OUTER', \
                        outerType must 'server' or 'client', but you input:%s" % self.outerType)
        return True


class CmdContent(object):
    code = None
    params = None

    def __init__(self, code, params):
        self.code = code
        self.params = params

    def get_code(self):
        return self.code

    def get_params(self):
        return self.params

    def get_text(self):
        raise Exception("This is cmd content, can't get text")


class TextContent(object):
    text = None
    compress = None

    def __init__(self, text, compress=False):
        self.text = text
        self.compress = compress

    def is_compress(self):
        return self.compress

    def get_code(self):
        raise Exception("This is text content, can't get code")

    def get_text(self):
        return self.text

    def get_params(self):
        raise Exception("This is text content, can't get params")


class Body(object):
    """
    messageType: 消息类型   必传
        cmd: 指令消息， json小数据传输
        text: 文本消息， 大数据长消息， 尽量避免大数据传输
    content: 消息内容  字典  必传
    """
    bodyData = None
    messageType = None
    content = None

    def __init__(self, body_data):
        self.bodyData = body_data

    def get_body_data(self):
        return self.bodyData

    def is_valid(self):
        if "messageType" not in self.bodyData.keys():
            raise Exception("Key 'messageType' not exist, but must input")
        else:
            self.messageType = self.bodyData['messageType']
        if "content" not in self.bodyData.keys():
            raise Exception("Key 'content' not exist, but must input")
        else:
            content = self.bodyData['content']
            if self.messageType == "cmd":
                if "code" not in content.keys():
                    raise Exception("Key 'code' not exist in content, but must input")
                if "params" not in content.keys():
                    raise Exception("Key 'params' not exist in content, but must input")
                self.content = CmdContent(content['code'], content['params'])
            elif self.messageType == "text":
                if "text" not in content.keys():
                    raise Exception("Key 'text' not exist in content, but must input")
                if "compress" in content.keys():
                    self.content = TextContent(text=content['text'], compress=content['compress'])
                else:
                    self.content = TextContent(text=content['text'])
            else:
                raise Exception("Not standby this messageType")
        return True

    def get_params(self):
        return self.content.get_params()

    def get_text(self):
        return self.content.get_text()


class Message(object):
    header = None
    body = None
    isValid = False
    agentClient = None

    def __init__(self, header, body, agent_client):
        self.header = Header(header)
        self.body = Body(body)
        self.agentClient = agent_client

    def is_valid(self):
        """
        详细参考文档： https://yuque.antfin-inc.com/zg1myq/ge9tsy/oc7ydw#JFG7H
        """
        self.isValid = self.header.is_valid() and self.body.is_valid()
        return self.isValid

    def get_body(self):
        if self.isValid:
            return self.body
        else:
            raise Exception("message not initial, please check valid")

    def get_content(self):
        if self.isValid:
            return self.body.content
        else:
            raise Exception("message not initial, please check valid")

    def get_message_type(self):
        if self.isValid:
            return self.body.messageType
        else:
            raise Exception("message not initial, please check valid")

    def get_cmd_params(self):
        if self.isValid:
            return self.body.content.get_params()
        else:
            raise Exception("message not initial, please check valid")

    def get_cmd_code(self):
        if self.isValid:
            return self.body.content.get_code()
        else:
            raise Exception("message not initial, please check valid")

    def get_content_text(self):
        if self.isValid:
            return self.body.content.get_text()
        else:
            raise Exception("message not initial, please check valid")

    def get_message(self):
        if self.isValid:
            return {
                "header": self.header.get_header_data(),
                "body": self.body.get_body_data()
            }
        else:
            raise Exception("message not initial, please check valid")

    def get_package_string(self):
        if self.isValid:
            return "5%s" % (json.dumps(self.get_message()))
        else:
            raise Exception("message not initial, please check valid")

    def process(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/19 9:46 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : messageClass.py
# @Software: PyCharm
# @Desc    :
from agent.packages.message.receive.finishWork import FinishWork
from agent.packages.message.receive.startWork import StartWork

defaultReceiveMessageClassList = {
    "START_WORK": StartWork,
    "FINISH_WORK": FinishWork
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/18 9:20 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : initializedAck.py
# @Software: PyCharm
# @Desc    :

from agent.packages.message.base import Message
from agent.tools import build_msg_id


class InitializedAck(Message):

    def __init__(self, agent_client):
        msgId = build_msg_id()
        header = {
            "msgId": msgId,
            "sender": agent_client.model_name,
            "msgTarget": "AGENT",
        }
        body = {
            "messageType": "cmd",
            "content": {
                "code": "INITIALIZED_ACK",
                "params": {}
            }
        }
        super(InitializedAck, self).__init__(header, body, agent_client)

    def process(self):
        self.agentClient.send_message(self.get_package_string())





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/5/20 12:30 下午
# @Author  : shuming.wsm
# @Site    : 
# @File    : testReceiveMessage.py
# @Software: PyCharm
# @Desc    :
from agent.packages.message.base import Message


class DemoReceiveMessage(Message):

    def process(self):
        print("this is demo receive message:%s" % self.get_cmd_params())
